@@ -76,15 +76,14 @@ class KeepAlive(Thread):
 
 
 class Client(object):
-
     """
     High level client to connect to an OPC-UA server.
 
     This class makes it easy to connect and browse address space.
-    It attemps to expose as much functionality as possible
-    but if you want more flexibility it is possible and adviced to
-    use UaClient object, available as self.uaclient
-    which offers the raw OPC-UA services interface.
+    It attempts to expose as much functionality as possible
+    but if you want more flexibility it is possible and advised to
+    use the UaClient object, available as self.uaclient, which offers
+    the raw OPC-UA services interface.
     """
 
     def __init__(self, url, timeout=4):
@@ -159,7 +158,7 @@ class Client(object):
         """
         Set SecureConnection mode. String format:
         Policy,Mode,certificate,private_key[,server_private_key]
-        where Policy is Basic128Rsa15 or Basic256,
+        where Policy is Basic128Rsa15, Basic256 or Basic256Sha256,
             Mode is Sign or SignAndEncrypt
             certificate, private_key and server_private_key are
                 paths to .pem or .der files
@@ -288,7 +287,10 @@ class Client(object):
         Send OPC-UA hello to server
         """
         ack = self.uaclient.send_hello(self.server_url.geturl(), self.max_messagesize, self.max_chunkcount)
-        # FIXME check ack
+
+        # TODO: Handle ua.UaError
+        if isinstance(ack, ua.UaStatusCodeError):
+            raise ack
 
     def open_secure_channel(self, renew=False):
         """
@@ -316,26 +318,6 @@ class Client(object):
         params.EndpointUrl = self.server_url.geturl()
         return self.uaclient.get_endpoints(params)
 
-    def register_server(self, server, discovery_configuration=None):
-        """
-        register a server to discovery server
-        if discovery_configuration is provided, the newer register_server2 service call is used
-        """
-        serv = ua.RegisteredServer()
-        serv.ServerUri = server.application_uri
-        serv.ProductUri = server.product_uri
-        serv.DiscoveryUrls = [server.endpoint.geturl()]
-        serv.ServerType = server.application_type
-        serv.ServerNames = [ua.LocalizedText(server.name)]
-        serv.IsOnline = True
-        if discovery_configuration:
-            params = ua.RegisterServer2Parameters()
-            params.Server = serv
-            params.DiscoveryConfiguration = discovery_configuration
-            return self.uaclient.register_server2(params)
-        else:
-            return self.uaclient.register_server(serv)
-
     def find_servers(self, uris=None):
         """
         send a FindServer request to the server. The answer should be a list of
@@ -588,3 +570,34 @@ class Client(object):
         This enums will be available in ua module
         """
         return load_enums(self)
+
+    def register_nodes(self, nodes):
+        """
+        Register nodes for faster read and write access (if supported by server)
+        Rmw: This call modifies the nodeid of the nodes, the original nodeid is
+        available as node.basenodeid
+        """
+        nodeids = [node.nodeid for node in nodes]
+        nodeids = self.uaclient.register_nodes(nodeids)
+        for node, nodeid in zip(nodes, nodeids):
+            node.basenodeid = node.nodeid
+            node.nodeid = nodeid
+        return nodes
+
+    def unregister_nodes(self, nodes):
+        """
+        Unregister nodes
+        """
+        nodeids = [node.nodeid for node in nodes]
+        self.uaclient.unregister_nodes(nodeids)
+        for node in nodes:
+            node.nodeid = node.basenodeid
+            node.basenodeid = None
+
+    def get_values(self, nodes):
+        """
+        Read the value of multiple nodes in one roundtrip.
+        """
+        nodes = [node.nodeid for node in nodes]
+        results = self.uaclient.get_attribute(nodes, ua.AttributeIds.Value)
+        return [result.Value.Value for result in results]
