@@ -4,6 +4,7 @@ Low level binary client
 
 import logging
 import socket
+import errno
 from threading import Thread, Lock
 from concurrent.futures import Future
 from functools import partial
@@ -109,7 +110,8 @@ class UASocketClient(object):
         elif isinstance(msg, ua.Acknowledge):
             self._call_callback(0, msg)
         elif isinstance(msg, ua.ErrorMessage):
-            self.logger.warning("Received an error: %s", msg)
+            self.logger.fatal("Received an error: %s", msg)
+            self._call_callback(0, ua.UaStatusCodeError(msg.Error.value))
         else:
             raise ua.UaError("Unsupported message type: %s", msg)
 
@@ -136,8 +138,11 @@ class UASocketClient(object):
         connect to server socket and start receiving thread
         """
         self.logger.info("opening connection")
-        sock = socket.create_connection((host, port))
-        # nodelay ncessary to avoid packing in one frame, some servers do not like it
+        # Create socket with timeout for initial connection
+        sock = socket.create_connection((host, port), timeout=self.timeout)
+        # set to blocking mode again
+        sock.settimeout(None)
+        # nodelay necessary to avoid packing in one frame, some servers do not like it
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         self._socket = ua.utils.SocketWrapper(sock)
         self.start()
@@ -145,11 +150,18 @@ class UASocketClient(object):
     def disconnect_socket(self):
         self.logger.info("Request to close socket received")
         self._do_stop = True
-        self._socket.socket.shutdown(socket.SHUT_RDWR)
+        try:
+            self._socket.socket.shutdown(socket.SHUT_RDWR)
+        except (socket.error, OSError) as exc:
+            if exc.errno in (errno.ENOTCONN, errno.EBADF):
+                pass # Socket is not connected, so can't send FIN packet.
+            else:
+                raise
         self._socket.socket.close()
         self.logger.info("Socket closed, waiting for receiver thread to terminate...")
         if self._thread and self._thread.is_alive():
             self._thread.join()
+        self._callbackmap.clear()
         self.logger.info("Done closing socket: Receiving thread terminated, socket disconnected")
 
     def send_hello(self, url, max_messagesize=0, max_chunkcount=0):
@@ -186,13 +198,20 @@ class UASocketClient(object):
         """
         self.logger.info("close_secure_channel")
         request = ua.CloseSecureChannelRequest()
-        future = self._send_request(request, message_type=ua.MessageType.SecureClose)
-        with self._lock:
-            # don't expect any more answers
-            future.cancel()
-            self._callbackmap.clear()
+        try:
+            future = self._send_request(request, message_type=ua.MessageType.SecureClose)
+            with self._lock:
+                # some servers send a response here, most do not ... so we ignore
+                future.cancel()
+        except (socket.error, OSError) as exc:
+            if exc.errno in (errno.ENOTCONN, errno.EBADF):
+                # Socket is closed, so can't send CloseSecureChannelRequest.
+                self.logger.warning("close_secure_channel() failed: socket already closed")
+            else:
+                raise
 
-        # some servers send a response here, most do not ... so we ignore
+    def is_secure_channel_open(self):
+        return self._connection.is_open()
 
 
 class UaClient(object):
@@ -264,6 +283,9 @@ class UaClient(object):
 
     def close_session(self, deletesubscriptions):
         self.logger.info("close_session")
+        # Bail out if we don't have an open server-channel to unsubscribe ourself.
+        if not self._uasocket.is_secure_channel_open():
+            return
         request = ua.CloseSessionRequest()
         request.DeleteSubscriptions = deletesubscriptions
         data = self._uasocket.send_request(request)
@@ -387,24 +409,30 @@ class UaClient(object):
         response.ResponseHeader.ServiceResult.check()
         return response.Results
 
-    def create_subscription(self, params, callback):
+    def create_subscription(self, params, publish_callback, ready_callback=None):
         self.logger.info("create_subscription")
         request = ua.CreateSubscriptionRequest()
         request.Parameters = params
         resp_fut = Future()
-        mycallbak = partial(self._create_subscription_callback, callback, resp_fut)
+        mycallbak = partial(self._create_subscription_callback, publish_callback, ready_callback, resp_fut)
         self._uasocket.send_request(request, mycallbak)
         return resp_fut.result(self._timeout)
 
-    def _create_subscription_callback(self, pub_callback, resp_fut, data_fut):
+    def _create_subscription_callback(self, pub_callback, ready_callback, resp_fut, data_fut):
         self.logger.info("_create_subscription_callback")
         data = data_fut.result()
         response = struct_from_binary(ua.CreateSubscriptionResponse, data)
         self.logger.debug(response)
         response.ResponseHeader.ServiceResult.check()
+        if ready_callback:
+            ready_callback(response)
         self._publishcallbacks[response.Parameters.SubscriptionId] = pub_callback
         resp_fut.set_result(response.Parameters)
 
+    def registered_subscriptions(self):
+        """Get all subscriptions we know about"""
+        return [cb.__self__ for cb in self._publishcallbacks.values()]
+
     def delete_subscriptions(self, subscriptionids):
         self.logger.info("delete_subscription")
         request = ua.DeleteSubscriptionsRequest()
@@ -572,3 +600,36 @@ class UaClient(object):
         self.logger.debug(response)
         response.ResponseHeader.ServiceResult.check()
         return response.Results
+
+    def register_nodes(self, nodes):
+        self.logger.info("register_nodes")
+        request = ua.RegisterNodesRequest()
+        request.Parameters.NodesToRegister = nodes
+        data = self._uasocket.send_request(request)
+        response = struct_from_binary(ua.RegisterNodesResponse, data)
+        self.logger.debug(response)
+        response.ResponseHeader.ServiceResult.check()
+        return response.Parameters.RegisteredNodeIds
+
+    def unregister_nodes(self, nodes):
+        self.logger.info("unregister_nodes")
+        request = ua.UnregisterNodesRequest()
+        request.Parameters.NodesToUnregister = nodes
+        data = self._uasocket.send_request(request)
+        response = struct_from_binary(ua.UnregisterNodesResponse, data)
+        self.logger.debug(response)
+        response.ResponseHeader.ServiceResult.check()
+        # nothing to return for this service
+
+    def get_attribute(self, nodes, attr):
+        self.logger.info("get_attribute")
+        request = ua.ReadRequest()
+        for node in nodes:
+            rv = ua.ReadValueId()
+            rv.NodeId = node
+            rv.AttributeId = attr
+            request.Parameters.NodesToRead.append(rv)
+        data = self._uasocket.send_request(request)
+        response = struct_from_binary(ua.ReadResponse, data)
+        response.ResponseHeader.ServiceResult.check()
+        return response.Results
