@@ -15,10 +15,11 @@ from opcua import ua
 from opcua.server.binary_server_asyncio import BinaryServer
 from opcua.server.internal_server import InternalServer
 from opcua.server.event_generator import EventGenerator
+from opcua.server.user_manager import UserManager
+from opcua.server.discovery_service import LocalDiscoveryService
 from opcua.common.node import Node
 from opcua.common.subscription import Subscription
 from opcua.common.manage_nodes import delete_nodes
-from opcua.client.client import Client
 from opcua.crypto import security_policies
 from opcua.common.event_objects import BaseEvent
 from opcua.common.shortcuts import Shortcuts
@@ -86,12 +87,8 @@ class Server(object):
         if iserver is not None:
             self.iserver = iserver
         else:
-            self.iserver = InternalServer(shelffile)
+            self.iserver = InternalServer(shelffile = shelffile, parent = self)
         self.bserver = None
-        self._discovery_clients = {}
-        self._discovery_period = 60
-        self.certificate = None
-        self.private_key = None
         self._policies = []
         self.nodes = Shortcuts(self.iserver.isession)
 
@@ -114,14 +111,15 @@ class Server(object):
 
 
         # enable all endpoints by default
+        self.certificate = None
+        self.private_key = None
+        self.user_manager = UserManager(parent = self)
         self._security_policy = [
                         ua.SecurityPolicyType.NoSecurity,
-                        ua.SecurityPolicyType.Basic128Rsa15_SignAndEncrypt,
-                        ua.SecurityPolicyType.Basic128Rsa15_Sign,
-                        ua.SecurityPolicyType.Basic256_SignAndEncrypt,
-                        ua.SecurityPolicyType.Basic256_Sign
+                        ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt,
+                        ua.SecurityPolicyType.Basic256Sha256_Sign
                                 ]
-        self._policyIDs = ["Anonymous", "Basic256", "Basic128", "Username"]
+        self._policyIDs = ["Anonymous", "Basic256Sha256", "Username"]
 
     def __enter__(self):
         self.start()
@@ -130,6 +128,10 @@ class Server(object):
     def __exit__(self, exc_type, exc_value, traceback):
         self.stop()
 
+    @property
+    def local_discovery_service(self):
+        return self.iserver.local_discovery_service
+
     def load_certificate(self, path):
         """
         load server certificate from file, either pem or der
@@ -163,6 +165,12 @@ class Server(object):
             uries.append(uri)
         ns_node.set_value(uries)
 
+    def get_application_uri(self):
+        """
+        Get application/server URI.
+        """
+        return self._application_uri
+
     def find_servers(self, uris=None):
         """
         find_servers. mainly implemented for symmetry with client
@@ -172,50 +180,13 @@ class Server(object):
         params = ua.FindServersParameters()
         params.EndpointUrl = self.endpoint.geturl()
         params.ServerUris = uris
-        return self.iserver.find_servers(params)
-
-    def register_to_discovery(self, url="opc.tcp://localhost:4840", period=60):
-        """
-        Register to an OPC-UA Discovery server. Registering must be renewed at
-        least every 10 minutes, so this method will use our asyncio thread to
-        re-register every period seconds
-        if period is 0 registration is not automatically renewed
-        """
-        # FIXME: have a period per discovery
-        if url in self._discovery_clients:
-            self._discovery_clients[url].disconnect()
-        self._discovery_clients[url] = Client(url)
-        self._discovery_clients[url].connect()
-        self._discovery_clients[url].register_server(self)
-        self._discovery_period = period
-        if period:
-            self.iserver.loop.call_soon(self._renew_registration)
-
-    def unregister_to_discovery(self, url="opc.tcp://localhost:4840"):
-        """
-        stop registration thread
-        """
-        # FIXME: is there really no way to deregister?
-        self._discovery_clients[url].disconnect()
-
-    def _renew_registration(self):
-        for client in self._discovery_clients.values():
-            client.register_server(self)
-            self.iserver.loop.call_later(self._discovery_period, self._renew_registration)
-
-    def get_client_to_discovery(self, url="opc.tcp://localhost:4840"):
-        """
-        Create a client to discovery server and return it
-        """
-        client = Client(url)
-        client.connect()
-        return client
+        return self.local_discovery_service.find_servers(params)
 
     def allow_remote_admin(self, allow):
         """
         Enable or disable the builtin Admin user from network clients
         """
-        self.iserver.allow_remote_admin = allow
+        self.user_manager.allow_remote_admin = allow
 
     def set_endpoint(self, url):
         self.endpoint = urlparse(url)
@@ -231,17 +202,14 @@ class Server(object):
 
                 security_policy = [
                             ua.SecurityPolicyType.NoSecurity,
-                            ua.SecurityPolicyType.Basic128Rsa15_SignAndEncrypt,
-                            ua.SecurityPolicyType.Basic128Rsa15_Sign,
-                            ua.SecurityPolicyType.Basic256_SignAndEncrypt,
-                            ua.SecurityPolicyType.Basic256_Sign
+                            ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt,
+                            ua.SecurityPolicyType.Basic256Sha256_Sign
                                 ]
 
             E.g. to limit the number of endpoints and disable no encryption:
 
                 set_security_policy([
-                            ua.SecurityPolicyType.Basic128Rsa15_SignAndEncrypt
-                            ua.SecurityPolicyType.Basic256_SignAndEncrypt])
+                            ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt])
 
         """
         self._security_policy = security_policy
@@ -249,14 +217,14 @@ class Server(object):
     def set_security_IDs(self, policyIDs):
         """
             Method setting up the security endpoints for identification
-            of clients. During server object initialization, all possible 
+            of clients. During server object initialization, all possible
             endpoints are enabled:
 
-            self._policyIDs = ["Anonymous", "Basic256", "Basic128", "Username"]
+            self._policyIDs = ["Anonymous", "Basic256Sha256", "Username"]
 
             E.g. to limit the number of IDs and disable anonymous clients:
 
-                set_security_policy(["Basic256"])
+                set_security_policy(["Basic256Sha256"])
 
             (Implementation for ID check is currently not finalized...)
 
@@ -277,34 +245,18 @@ class Server(object):
             if ua.SecurityPolicyType.NoSecurity in self._security_policy:
                 self.logger.warning("Creating an open endpoint to the server, although encrypted endpoints are enabled.")
 
-            if ua.SecurityPolicyType.Basic128Rsa15_SignAndEncrypt in self._security_policy:
-                self._set_endpoints(security_policies.SecurityPolicyBasic128Rsa15,
+            if ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt in self._security_policy:
+                self._set_endpoints(security_policies.SecurityPolicyBasic256Sha256,
                                     ua.MessageSecurityMode.SignAndEncrypt)
-                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic128Rsa15,
+                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic256Sha256,
                                                                ua.MessageSecurityMode.SignAndEncrypt,
                                                                self.certificate,
                                                                self.private_key)
                                      )
-            if ua.SecurityPolicyType.Basic128Rsa15_Sign in self._security_policy:
-                self._set_endpoints(security_policies.SecurityPolicyBasic128Rsa15,
+            if ua.SecurityPolicyType.Basic256Sha256_Sign in self._security_policy:
+                self._set_endpoints(security_policies.SecurityPolicyBasic256Sha256,
                                     ua.MessageSecurityMode.Sign)
-                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic128Rsa15,
-                                                               ua.MessageSecurityMode.Sign,
-                                                               self.certificate,
-                                                               self.private_key)
-                                     )
-            if ua.SecurityPolicyType.Basic256_SignAndEncrypt in self._security_policy:
-                self._set_endpoints(security_policies.SecurityPolicyBasic256,
-                                    ua.MessageSecurityMode.SignAndEncrypt)
-                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic256,
-                                                               ua.MessageSecurityMode.SignAndEncrypt,
-                                                               self.certificate,
-                                                               self.private_key)
-                                     )
-            if ua.SecurityPolicyType.Basic256_Sign in self._security_policy:
-                self._set_endpoints(security_policies.SecurityPolicyBasic256,
-                                    ua.MessageSecurityMode.Sign)
-                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic256,
+                self._policies.append(ua.SecurityPolicyFactory(security_policies.SecurityPolicyBasic256Sha256,
                                                                ua.MessageSecurityMode.Sign,
                                                                self.certificate,
                                                                self.private_key)
@@ -318,15 +270,9 @@ class Server(object):
             idtoken.TokenType = ua.UserTokenType.Anonymous
             idtokens.append(idtoken)
 
-        if "Basic256" in self._policyIDs:
+        if "Basic256Sha256" in self._policyIDs:
             idtoken = ua.UserTokenPolicy()
-            idtoken.PolicyId = 'certificate_basic256'
-            idtoken.TokenType = ua.UserTokenType.Certificate
-            idtokens.append(idtoken)
-
-        if "Basic128" in self._policyIDs:
-            idtoken = ua.UserTokenPolicy()
-            idtoken.PolicyId = 'certificate_basic128'
+            idtoken.PolicyId = 'certificate_basic256sha256'
             idtoken.TokenType = ua.UserTokenType.Certificate
             idtokens.append(idtoken)
 
@@ -378,8 +324,6 @@ class Server(object):
         """
         Stop server
         """
-        for client in self._discovery_clients.values():
-            client.disconnect()
         self.bserver.stop()
         self.iserver.stop()
 
@@ -453,14 +397,14 @@ class Server(object):
         uries = self.get_namespace_array()
         return uries.index(uri)
 
-    def get_event_generator(self, etype=None, source=ua.ObjectIds.Server):
+    def get_event_generator(self, etype=None, emitting_node=ua.ObjectIds.Server):
         """
         Returns an event object using an event type from address space.
         Use this object to fire events
         """
         if not etype:
             etype = BaseEvent()
-        return EventGenerator(self.iserver.isession, etype, source)
+        return EventGenerator(self.iserver.isession, etype, emitting_node=emitting_node)
 
     def create_custom_data_type(self, idx, name, basetype=ua.ObjectIds.BaseDataType, properties=None):
         if properties is None:
